@@ -1,17 +1,26 @@
 ;;; -*- lexical-binding: t -*-
 
-(require 'undercover)
-(undercover "*.el"
-            (:exclude "test-*.el"))
-
 (require 'ido)
-(require 'flx-ido)
 (require 'minibuf-eldef)
 (require 'ido-completing-read+)
 (require 'buttercup)
 (require 'cl-lib)
 (require 'with-simulated-input)
 
+;; Note: Currently unused, but potentially useful in the future
+(defun ido-cr+-maybe-chop (items elem)
+  "Like `ido-chop', but a no-op if ELEM is not in ITEMS.
+
+Normal `ido-chop' hangs infinitely in this case."
+  (cl-loop
+   with new-tail = ()
+   for remaining on items
+   for next = (car remaining)
+   if (equal next elem)
+   return (nconc remaining new-tail)
+   else collect next into new-tail
+   finally return items))
+
 (defun collection-as-function (collection)
   "Return a function equivalent to COLLECTION.
 
@@ -114,6 +123,17 @@ also accept a quoted list for the sake of convenience."
     (setq vars (eval vars)))
   `(mapc #'unshadow-var ',vars))
 
+(defmacro with-temp-info-buffer (&rest body)
+  "Create a temporary info buffer and exeluate BODY forms there."
+  (declare (indent 0))
+  `(let ((temp-bufname (generate-new-buffer-name " *temp-info*")))
+     (unwind-protect
+         (save-excursion
+           (info nil (generate-new-buffer-name " *temp-info*"))
+           ,@body)
+       (when (get-buffer temp-bufname)
+         (kill-buffer temp-bufname)))))
+
 (describe "Within the `ido-completing-read+' package"
 
   ;; Reset all of these variables to their standard values before each
@@ -133,7 +153,6 @@ also accept a quoted list for the sake of convenience."
       ido-confirm-unique-completion
       ido-enable-flex-matching
       ido-enable-dot-prefix
-      flx-ido-mode
       (minibuffer-electric-default-mode t)))
 
     ;; Suppress all messages during tests
@@ -184,6 +203,23 @@ also accept a quoted list for the sake of convenience."
          (ido-completing-read+ "Prompt: " '("blue" "yellow" "green") nil t))
        :to-equal ""))
 
+    (it "should work with `minibuffer-electric-default-mode'"
+      (let ((eldef-was-showing nil))
+        ;; No REQUIRE-MATCH, so electric default should not show
+        (with-simulated-input
+            '("blu DEL DEL DEL"
+              (setq eldef-was-showing minibuf-eldef-showing-default-in-prompt)
+              "RET")
+          (ido-completing-read+ "Prompt (default green): " '("blue" "yellow" "green")))
+        (expect eldef-was-showing :not :to-be-truthy)
+        ;; With REQUIRE-MATCH, so electric default should show
+        (with-simulated-input
+            '("blu DEL DEL DEL"
+              (setq eldef-was-showing minibuf-eldef-showing-default-in-prompt)
+              "RET")
+          (ido-completing-read+ "Prompt (default green): " '("blue" "yellow" "green") nil t))
+        (expect eldef-was-showing :to-be-truthy)))
+
     (it "should accept all the same forms of DEF as `completing-read-default'"
       ;; DEF in COLLECTION
       (expect
@@ -565,109 +601,7 @@ also accept a quoted list for the sake of convenience."
            (with-simulated-input "eee C-SPC aaa C-u C-SPC ccc C-u C-SPC ggg RET"
              (ido-completing-read+
               "Pick: " (collection-as-function collection) nil t nil nil (car collection)))
-           :to-equal "bbb-eee-ggg")))
-
-      (describe "with flx-ido-mode"
-        (before-each
-          (flx-ido-mode 1)
-          (flx-ido-reset))
-
-        (it "should allow selection of dynamically-added completions"
-          (expect
-           (with-simulated-input "hello-w RET"
-             (ido-completing-read+ "Say something: " my-dynamic-collection))
-           :to-equal "hello-world")
-          (expect 'ido-cr+-update-dynamic-collection
-                  :to-have-been-called))
-
-        (it "should allow ido flex-matching of dynamically-added completions"
-          (expect
-           (with-simulated-input "hello-ld RET"
-             (ido-completing-read+ "Say something: " my-dynamic-collection))
-           :to-equal
-           "hello-world")
-          (expect 'ido-cr+-update-dynamic-collection
-                  :to-have-been-called))
-
-        (it "should do a dynamic update when pressing TAB"
-          (expect
-           (with-simulated-input "h TAB -ld RET"
-             (ido-completing-read+ "Say something: " my-dynamic-collection))
-           :to-equal
-           "hello-world")
-          (expect 'ido-cr+-update-dynamic-collection
-                  :to-have-been-called))
-
-        (it "should do a dynamic update when idle"
-          (expect
-           (with-simulated-input
-               '("h"
-                 (wsi-simulate-idle-time (1+ ido-cr+-dynamic-update-idle-time))
-                 "-ld RET")
-             (ido-completing-read+ "Say something: " my-dynamic-collection))
-           :to-equal
-           "hello-world")
-          (expect 'ido-cr+-update-dynamic-collection
-                  :to-have-been-called))
-
-        (it "should do a dynamic update when there is only one match remaining"
-          (expect
-           (with-simulated-input "hell-ld RET"
-             (ido-completing-read+ "Say something: " my-dynamic-collection))
-           :to-equal
-           "hello-world")
-          (expect 'ido-cr+-update-dynamic-collection
-                  :to-have-been-called))
-
-        (it "should not exit with a unique match if new matches are dynamically added"
-          (expect
-           (with-simulated-input '("hell TAB -ld RET")
-             (ido-completing-read+ "Say something: " my-dynamic-collection))
-           :to-equal
-           "hello-world")
-          (expect 'ido-cr+-update-dynamic-collection
-                  :to-have-been-called))
-
-        (it "should exit with a match that is still unique after dynamic updating"
-          (expect
-           (with-simulated-input '("helic TAB")
-             (ido-completing-read+ "Say something: " my-dynamic-collection))
-           :to-equal
-           "helicopter")
-          (expect 'ido-cr+-update-dynamic-collection
-                  :to-have-been-called))
-
-        (it "should respect `ido-restrict-to-matches' when doing dynamic updates"
-          (let ((collection
-                 (list "aaa-ddd-ggg" "aaa-eee-ggg" "aaa-fff-ggg"
-                       "bbb-ddd-ggg" "bbb-eee-ggg" "bbb-fff-ggg"
-                       "ccc-ddd-ggg" "ccc-eee-ggg" "ccc-fff-ggg"
-                       "aaa-ddd-hhh" "aaa-eee-hhh" "aaa-fff-hhh"
-                       "bbb-ddd-hhh" "bbb-eee-hhh" "bbb-fff-hhh"
-                       "ccc-ddd-hhh" "ccc-eee-hhh" "ccc-fff-hhh"
-                       "aaa-ddd-iii" "aaa-eee-iii" "aaa-fff-iii"
-                       "bbb-ddd-iii" "bbb-eee-iii" "bbb-fff-iii"
-                       "ccc-ddd-iii" "ccc-eee-iii" "ccc-fff-iii")))
-            ;; Test the internal function
-            (expect
-             (ido-cr+-apply-restrictions
-              collection
-              (list (cons nil "bbb")
-                    (cons nil "eee")))
-             :to-equal '("bbb-eee-ggg" "bbb-eee-hhh" "bbb-eee-iii"))
-
-            ;; First verify it without a dynamic collection
-            (expect
-             (with-simulated-input "eee C-SPC bbb C-SPC ggg RET"
-               (ido-completing-read+
-                "Pick: " collection nil t nil nil (car collection)))
-             :to-equal "bbb-eee-ggg")
-            ;; Now test the same with a dynamic collection
-            (expect
-             (with-simulated-input "eee C-SPC bbb C-SPC ggg RET"
-               (ido-completing-read+
-                "Pick: " (collection-as-function collection) nil t nil nil (car collection)))
-             :to-equal "bbb-eee-ggg")))))
+           :to-equal "bbb-eee-ggg"))))
 
     (describe "with unusual inputs"
       (it "should accept a COLLECTION of symbols"
@@ -1011,10 +945,62 @@ also accept a quoted list for the sake of convenience."
           (expect
            (with-simulated-input "RET"
              (ido-completing-read+ "Prompt: " 'def-nil-collection nil t))
-           :to-equal "blue"))))))
+           :to-equal "blue"))))
 
-;; (defun ido-cr+-run-all-tests ()
-;;   (interactive)
-;;   (ert "^ido-cr\\+-"))
+    ;; Test is currently disabled pending additional information
+    (xit "should not hang or error when deleting characters in `org-refile' (issue #152)"
+      (expect
+       (progn
+         (ido-ubiquitous-mode 1)
+         (save-excursion
+           (with-temp-buffer
+             (org-mode)
+             (insert (s-trim "
+    * Heading 1
+    ** Subheading 1.1
+    ** Subheading 1.2
+    ** Subheading 1.3
+    * Heading 2
+    * Heading 3
+    "))
+             (goto-char (point-max))
+             ;; TODO Figure out what else needs to be set up to call
+             ;; `org-refile'
+             (with-simulated-input
+                 "Heading DEL DEL DEL DEL DEL RET"
+               (command-execute 'org-refile)))))
+       :not :to-throw)))
+
+  (describe "regressions should not occur for"
+    (it "issue #151: should not hang or error when cycling matches in `Info-menu'"
+      (expect
+       (progn
+         (ido-ubiquitous-mode 1)
+         (with-temp-info-buffer
+           (with-simulated-input
+               '("emacs"
+                 (ido-next-match)
+                 (wsi-simulate-idle-time 5)
+                 (ido-next-match)
+                 (wsi-simulate-idle-time 5)
+                 (ido-next-match)
+                 (wsi-simulate-idle-time 5)
+                 (ido-next-match)
+                 (wsi-simulate-idle-time 5)
+                 "RET")
+             (command-execute 'Info-menu))))
+       :not :to-throw))
+
+    (it "issue #153: should preserve the selected item when doing a deferred dynamic update"
+      (expect
+       (with-simulated-input
+           '("Emacs"
+             (ido-next-match)
+             (wsi-simulate-idle-time 5)
+             "RET")
+         (ido-completing-read+
+          "Choose: "
+          (collection-as-function '("Emacs" "Emacs A" "Emacs B" "Emacs C"))))
+       :to-equal "Emacs A"))))
 
 ;;; test-ido-completing-read+.el ends here
